Texture2D texUI : register(t0);
Texture3D texVolume : register(t1);
Texture1D texTransfer : register(t2);
SamplerState samUI : register(s0);
SamplerState samVolume : register(s1);
SamplerState samTransfer : register(s2);

/*cbuffer cbNeverChanges : register(b0)
{
};*/

cbuffer cbChangeOnResize : register(b1)
{
    matrix matProjection;
	matrix matOrtho;
};

cbuffer cbChangesEveryFrameVolume : register(b2)
{
    matrix matView;
	float4 vecView;
};

cbuffer cbChangesEveryFrameUI : register(b3)
{
	matrix matUITransform;
};

cbuffer cbChangesOnVolume : register(b4)
{
	float4 dd;	// 
};


	// shaders UI

struct VS_Out_STex
{
	float4 position : SV_POSITION;
	float2 tex : TEXCOORD;
};

VS_Out_STex VS_STex(float4 pos : POSITION, float2 tex : TEXCOORD)
{
	VS_Out_STex output = (VS_Out_STex) 0;
	output.position = mul(pos, matUITransform);
	output.position = mul(output.position, matOrtho);
	output.tex = tex;
	return output;
}

float4 PS_STex(VS_Out_STex psinput) : SV_Target
{
	return texUI.Sample(samUI, psinput.tex);
}



struct VS_Out_SClr
{
	float4 position : SV_POSITION;
	float4 colour : COLOR;
};

VS_Out_SClr VS_SClr(float4 pos : POSITION, float4 colour : COLOR)
{
	VS_Out_SClr output = (VS_Out_SClr) 0;
	output.position = mul(pos, matUITransform);
	output.position = mul(output.position, matOrtho);
	output.colour = colour;
	return output;
}

float4 PS_SClr(VS_Out_SClr psinput) : SV_Target
{
	return psinput.colour;
}



	// shaders volume

struct VS_Out_Volume
{
	float4 position : SV_POSITION;
	float3 tex : TEXCOORD;
};

VS_Out_Volume VS_Volume(float4 pos : POSITION, float3 tex : TEXCOORD)
{
	VS_Out_Volume output = (VS_Out_Volume) 0;
	//output.position = mul(pos, matView);
	//output.position = mul(output.position, matProjection);
	//output.position = pos;
	output.position = mul(pos, matProjection);
	output.tex = tex;
	return output;
}

float4 PS_Volume(VS_Out_Volume psinput) : SV_Target
{
	// shader constants
	const float3 l = {0.69336127435063470484335227478596f, 0.69336127435063470484335227478596f, 0.69336127435063470484335227478596f};	// light vector

	const float3 n = 
	{ 
		(texVolume.Sample(samVolume, float3(psinput.tex.x+dd.x, psinput.tex.y, psinput.tex.z)).r - 
		 texVolume.Sample(samVolume, float3(psinput.tex.x-dd.x, psinput.tex.y, psinput.tex.z)).r) / (2*dd.x),
				
		(texVolume.Sample(samVolume, float3(psinput.tex.x, psinput.tex.y+dd.y, psinput.tex.z)).r - 
		 texVolume.Sample(samVolume, float3(psinput.tex.x, psinput.tex.y-dd.y, psinput.tex.z)).r) / (2*dd.y),
				
		(texVolume.Sample(samVolume, float3(psinput.tex.x, psinput.tex.y, psinput.tex.z+dd.z)).r - 
		 texVolume.Sample(samVolume, float3(psinput.tex.x, psinput.tex.y, psinput.tex.z-dd.z)).r) / (2*dd.z)
	};

	if(length(n) < 0.05f)
	{
		return float4(0.0f, 0.0f, 0.0f, 0.0f);
	}
	else
	{
		normalize(n);
		float den = texVolume.Sample(samVolume, psinput.tex).r; 
		float4 rgba = texTransfer.Sample(samTransfer, den);
		float3 rgb = rgba.rgb;

			// blinn - phong

		float i = saturate(dot(l, n)); 

			//diffuse
		float3 bp = i * rgb * 0.5; // / distance; distance = 1
	
		float3 h = normalize(l + vecView.xyz);
			//specular
		i = pow(saturate(dot(n, h)), 10.0f);	
		bp += i * rgb * 0.5;

		return float4(bp, rgba.a);
	}
}